@@ -75,6 +75,11 @@ gsm_util_get_app_dirs ()
 
         dirs = g_ptr_array_new ();
 
+        g_ptr_array_add (dirs,
+			 g_build_filename (g_get_user_data_dir (),
+					   "applications",
+					   NULL));
+
         system_data_dirs = g_get_system_data_dirs ();
         for (i = 0; system_data_dirs[i]; i++) {
                 g_ptr_array_add (dirs,
@@ -135,7 +140,7 @@ gsm_util_init_error (gboolean    fatal,
         if (!gdk_display_get_default ()) {
                 if (!gtk_init_check (NULL, NULL)) {
                         /* Oh well, no X for you! */
-                        g_printerr (_("Unable to start login session (and unable connect to the X server)"));
+                        g_printerr (_("Unable to start login session (and unable to connect to the X server)"));
                         g_printerr ("%s", msg);
                         exit (1);
                 }
